@@ -10,20 +10,85 @@ Public chart functions
 """
 
 
-def create_chart(reference: str, centile_format: Union[str, list] = COLE_TWO_THIRDS_SDS_NINE_CENTILES, measurement_method: str = HEIGHT, sex: str = FEMALE):
+def create_chart(reference: str, centile_format: Union[str, list] = COLE_TWO_THIRDS_SDS_NINE_CENTILES, measurement_method: str = HEIGHT, sex: str = FEMALE, is_sds=False):
     """
     Global method - return chart for measurement_method, sex and reference
     """
     
     if reference == UK_WHO:
-        return create_uk_who_chart(measurement_method=measurement_method, sex=sex, centile_format=centile_format)
+        return create_uk_who_chart(measurement_method=measurement_method, sex=sex, centile_format=centile_format, is_sds=is_sds)
     elif reference == TURNERS:
-        return create_turner_chart(centile_format=centile_format)
+        return create_turner_chart(centile_format=centile_format, is_sds=is_sds)
     elif reference == TRISOMY_21:
-        return create_trisomy_21_chart(measurement_method=measurement_method, sex=sex, centile_format=centile_format)
+        return create_trisomy_21_chart(measurement_method=measurement_method, sex=sex, centile_format=centile_format, is_sds=is_sds)
     else:
         print("No reference data returned. Is there a spelling mistake in your reference?")
 
+def generate_custom_sds_line(reference: str, measurement_method: str, sex: str, custom_sds: float):
+    # Public function that returns a custom SDS line
+    # the centile reference data
+    
+    ##
+    # iterate through the 4 references that make up UK-WHO
+    # There will be a list for each one. For the other references ther will be only one list
+    ##
+
+    # all data for a given reference are stored here: this is returned to the user
+    reference_data = []
+
+    custom_centile = centile(custom_sds)
+
+    if reference == UK_WHO:
+        for reference_index, reference in enumerate(UK_WHO_REFERENCES):
+            # the centile reference data
+            lms_array_for_measurement=select_reference_data_for_uk_who_chart(
+                uk_who_reference=reference, 
+                measurement_method=measurement_method, 
+                sex=sex)
+            centile_data=[]
+            try:
+                centile_data= build_centile_object(
+                    measurement_method=measurement_method,
+                    sex=sex,
+                    lms_array_for_measurement=lms_array_for_measurement,
+                    z=custom_sds,
+                    centile=custom_centile
+                )
+            except:
+                print("Could not generate centile data.")
+                centile_data=[]
+        # all data can now be tagged by reference_name and added to reference_data
+        reference_data.append({reference: centile_data})
+    else:
+        # get the reference data
+        lms_array_for_measurement=[]
+        try:
+            lms_array_for_measurement=select_reference_lms_data(
+                reference=reference,
+                measurement_method=measurement_method,
+                sex=sex
+            )
+        except:
+            print(f"It was not possible to retrieve {reference} data.")
+            lms_array_for_measurement=[]
+
+        try:
+            centile_data=[]
+            centile_data= build_centile_object(
+                    measurement_method=measurement_method,
+                    sex=sex,
+                    lms_array_for_measurement=lms_array_for_measurement,
+                    z=custom_sds,
+                    centile=custom_centile
+                )
+        except:
+            print("Could not generate sds data.")
+            centile_data=[]
+
+        reference_data.append({reference: centile_data})
+
+    return reference_data
+
 def generate_custom_centile(reference: str, measurement_method: str, sex: str, custom_centile: float)->list:
 
     # Public function that returns a custom centile line
@@ -287,26 +352,27 @@ def build_centile_object(measurement_method: str, sex: str, lms_array_for_measur
     return sex_list
 
 
-def create_uk_who_chart(measurement_method: str, sex: str, centile_format: Union[str, list] = COLE_TWO_THIRDS_SDS_NINE_CENTILES):
+def create_uk_who_chart(measurement_method: str, sex: str, centile_format: Union[str, list] = COLE_TWO_THIRDS_SDS_NINE_CENTILES, is_sds = False):
 
     # user selects which centile collection they want, for sex and measurement_method
     # If the Cole method is selected, conversion between centile and SDS
     # is different as SDS is rounded to the nearest 2/3
     # Cole method selection is stored in the cole_method flag.
     # If no parameter is passed, default is the Cole method
-    # Alternatively it is possible to pass a custom list of values
+    # Alternatively it is possible to pass a custom list of values - if the is_sds flag is False (default) these are centiles
 
-    centile_collection = []
+    centile_sds_collection = []
     cole_method = False
 
     if (type(centile_format) is list):
-        centile_collection = centile_format
+        centile_sds_collection = centile_format
     elif centile_format == COLE_TWO_THIRDS_SDS_NINE_CENTILES:
-        centile_collection = COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
+        centile_sds_collection = COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
         cole_method = True
+        is_sds=False
     else:
-        centile_collection = THREE_PERCENT_CENTILE_COLLECTION
-
+        centile_sds_collection = THREE_PERCENT_CENTILE_COLLECTION
+        is_sds=False
     ##
     # iterate through the 4 references that make up UK-WHO
     # There will be a list for each one
@@ -315,8 +381,6 @@ def create_uk_who_chart(measurement_method: str, sex: str, centile_format: Union
     # all data for a given reference are stored here: this is returned to the user
     reference_data = []
 
-    
-
     for reference_index, reference in enumerate(UK_WHO_REFERENCES):
         sex_list: dict = {}  # all the data for a given sex are stored here
         # For each reference we have 2 sexes
@@ -338,16 +402,21 @@ def create_uk_who_chart(measurement_method: str, sex: str, centile_format: Union
         except:
             lms_array_for_measurement = []
 
-        for centile_index, centile in enumerate(centile_collection):
+        for centile_index, centile_sds in enumerate(centile_sds_collection):
             # we must create a z for each requested centile
             # if the Cole 9 centiles were selected, these are rounded,
             # so conversion to SDS is different
             # Otherwise standard conversation of centile to z is used
             if cole_method:
-                z = rounded_sds_for_centile(centile)
+                z = rounded_sds_for_centile(centile_sds)
+                centile_value=centile_sds
             else:
-                z = sds_for_centile(centile)
-
+                if (is_sds):
+                    z=centile_sds
+                    centile_value=centile(centile_sds)
+                else:
+                    z = sds_for_centile(centile_sds)
+                    centile_value=centile_sds
             centile_data = []
 
             try:
@@ -356,18 +425,19 @@ def create_uk_who_chart(measurement_method: str, sex: str, centile_format: Union
                 # If this happens, an empty list is returned.
                 centile_data = generate_centile(
                     z=z,
-                    centile=centile,
+                    centile=centile_value,
                     measurement_method=measurement_method,
                     sex=sex,
                     lms_array_for_measurement=lms_array_for_measurement,
-                    reference=UK_WHO
+                    reference=UK_WHO,
+                    is_sds=is_sds
                 )
             except:
                 centile_data=None
             # Store this centile for a given measurement
             
             centiles.append({"sds": round(z * 100) / 100,
-                        "centile": centile, "data": centile_data})
+                        "centile": centile_value, "data": centile_data})
 
         # this is the end of the centile_collection for loop
         # All the centiles for this measurement, sex and reference are added to the measurements list
@@ -407,23 +477,25 @@ def create_uk_who_chart(measurement_method: str, sex: str, centile_format: Union
     """
 
 
-def create_turner_chart(centile_format: Union[str, list]):
+def create_turner_chart(centile_format: Union[str, list], is_sds=False):
    # user selects which centile collection they want
     # If the Cole method is selected, conversion between centile and SDS
     # is different as SDS is rounded to the nearest 2/3
     # Cole method selection is stored in the cole_method flag.
     # If no parameter is passed, default is the Cole method
 
-    centile_collection = []
+    centile_sds_collection = []
     cole_method = False
 
     if (type(centile_format) is list):
-        centile_collection = centile_format
+        centile_sds_collection = centile_format
     elif centile_format == COLE_TWO_THIRDS_SDS_NINE_CENTILES:
-        centile_collection = COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
+        centile_sds_collection = COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
         cole_method = True
+        is_sds=False
     else:
-        centile_collection = THREE_PERCENT_CENTILE_COLLECTION
+        centile_sds_collection = THREE_PERCENT_CENTILE_COLLECTION
+        is_sds=False
 
     # all data for a the reference are stored here: this is returned to the user
     reference_data = {}
@@ -444,16 +516,21 @@ def create_turner_chart(centile_format: Union[str, list]):
 
     centiles = []  # all generated centiles for a selected centile collection are stored here
 
-    for centile_index, centile in enumerate(centile_collection):
+    for centile_index, centile_sds in enumerate(centile_sds_collection):
         # we must create a z for each requested centile
         # if the Cole 9 centiles were selected, these are rounded,
         # so conversion to SDS is different
         # Otherwise standard conversation of centile to z is used
         if cole_method:
-            z = rounded_sds_for_centile(centile)
+            z = rounded_sds_for_centile(centile_sds)
+            centile_value=centile_sds
         else:
-            z = sds_for_centile(centile)
-
+            if is_sds:
+                z = centile_sds
+                centile_value = centile(centile_sds)
+            else:
+                z = sds_for_centile(centile_sds)
+                centile_value=centile_sds
         # Collect the LMS values from the correct reference
         lms_array_for_measurement = select_reference_data_for_turner(
             measurement_method=HEIGHT, sex=sex)
@@ -461,12 +538,17 @@ def create_turner_chart(centile_format: Union[str, list]):
         # Some data does not exist at all ages, so any error reflects missing data.
         # If this happens, an empty list is returned.
         try:
-            centile_data = generate_centile(z=z, centile=centile, measurement_method=HEIGHT,
-                                            sex=sex, lms_array_for_measurement=lms_array_for_measurement, reference=TURNERS)
+            centile_data = generate_centile(
+                z=z, 
+                centile=centile_value, 
+                measurement_method=HEIGHT,
+                sex=sex, 
+                lms_array_for_measurement=lms_array_for_measurement, reference=TURNERS,
+                is_sds=is_sds)
 
             # Store this centile for a given measurement
             centiles.append({"sds": round(z * 100) / 100,
-                            "centile": centile, "data": centile_data})
+                            "centile": centile_value, "data": centile_data})
         except Exception as e:
             print(e)
 
@@ -501,23 +583,25 @@ def create_turner_chart(centile_format: Union[str, list]):
     """
 
 
-def create_trisomy_21_chart(measurement_method: str, sex: str, centile_format: Union[str, list]):
+def create_trisomy_21_chart(measurement_method: str, sex: str, centile_format: Union[str, list], is_sds=False):
    # user selects which centile collection they want
     # If the Cole method is selected, conversion between centile and SDS
     # is different as SDS is rounded to the nearest 2/3
     # Cole method selection is stored in the cole_method flag.
     # If no parameter is passed, default is the Cole method
 
-    centile_collection = []
+    centile_sds_collection = []
     cole_method = False
 
     if (type(centile_format) is list):
-        centile_collection = centile_format
+        centile_sds_collection = centile_format
     elif centile_format == COLE_TWO_THIRDS_SDS_NINE_CENTILES:
-        centile_collection = COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
+        centile_sds_collection = COLE_TWO_THIRDS_SDS_NINE_CENTILE_COLLECTION
         cole_method = True
+        is_sds=False
     else:
-        centile_collection = THREE_PERCENT_CENTILE_COLLECTION
+        centile_sds_collection = THREE_PERCENT_CENTILE_COLLECTION
+        is_sds=False
 
     # all data for a the reference are stored here: this is returned to the user
     reference_data = {}
@@ -534,16 +618,21 @@ def create_trisomy_21_chart(measurement_method: str, sex: str, centile_format: U
 
     centiles = []  # all generated centiles for a selected centile collection are stored here
 
-    for centile_index, centile in enumerate(centile_collection):
+    for centile_index, centile_sds in enumerate(centile_sds_collection):
         # we must create a z for each requested centile
         # if the Cole 9 centiles were selected, these are rounded,
         # so conversion to SDS is different
         # Otherwise standard conversation of centile to z is used
         if cole_method:
-            z = rounded_sds_for_centile(centile)
+            z = rounded_sds_for_centile(centile_sds)
+            centile_value=centile_sds
         else:
-            z = sds_for_centile(centile)
-
+            if is_sds:
+                z = centile_sds
+                centile_value=centile(z)
+            else:
+                z = sds_for_centile(centile_sds)
+                centile_value=centile_sds
         # Collect the LMS values from the correct reference
         lms_array_for_measurement = select_reference_data_for_trisomy_21(
             measurement_method=measurement_method, sex=sex)
@@ -551,12 +640,18 @@ def create_trisomy_21_chart(measurement_method: str, sex: str, centile_format: U
         # Some data does not exist at all ages, so any error reflects missing data.
         # If this happens, an empty list is returned.
         try:    
-            centile_data = generate_centile(z=z, centile=centile, measurement_method=measurement_method,
-                                            sex=sex, lms_array_for_measurement=lms_array_for_measurement, reference=TRISOMY_21)
+            centile_data = generate_centile(
+                z=z, 
+                centile=centile_value, 
+                measurement_method=measurement_method,
+                sex=sex, 
+                lms_array_for_measurement=lms_array_for_measurement, 
+                reference=TRISOMY_21,
+                is_sds=is_sds)
 
             # Store this centile for a given measurement
             centiles.append({"sds": round(z * 100) / 100,
-                            "centile": centile, "data": centile_data})
+                            "centile": centile_value, "data": centile_data})
         except Exception as e:
             print(e)
 
